//***************************************************************************************************/
//  Board:                                    SDA                    SCL                    Level
//  Blue Pill, STM32F103xxxx boards.......... PB7                    PB6                    3.3v/5v
//***************************************************************************************************/

// izlazi komplementarnog PWM-a su na PA8 i PA9


#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define COLUMS           16
#define ROWS             2


#include <Thermistor.h>
#include <NTC_Thermistor.h>

#define SENSOR_PIN             PA3
#define REFERENCE_RESISTANCE   10000
#define NOMINAL_RESISTANCE     10000
#define NOMINAL_TEMPERATURE    25
#define B_VALUE                3950
#define STM32_ANALOG_RESOLUTION 4095

Thermistor* thermistor;



#define Frekvencija  35500                          // Za Mosfet u push pull izlazima 


#define Brojach     72000000/4/Frekvencija          // STM32F103 moze raditi na 48Mhz/72Mhz/128Mhz // PAZNJA !!!! PAZNJA !!!
#define DeadTime    20
#define MaxPwmVrednost    Brojach / 2 - DeadTime


int PWM_Vrednost = 1;


LiquidCrystal_I2C lcd(PCF8574_ADDR_A21_A11_A01, 4, 5, 6, 16, 11, 12, 13, 14, POSITIVE);


// ===============================================================================================
// ===============================================================================================

void SetDuty(int Duty)
{
    if(Duty > MaxPwmVrednost)
        Duty = MaxPwmVrednost;

    TIMER1_BASE->CCR1 = Brojach - Duty;
    TIMER1_BASE->CCR2 = Duty;  

    TIMER1_BASE->CR1 = 225;
    TIMER1_BASE->EGR = 1;
}

// ===============================================================================================
// ===============================================================================================

byte solar[8] =                                                             //ikona solar
{
  0b11111,0b10101,0b11111,0b10101,0b11111,0b10101,0b11111,0b00000
};

byte battery[8] =                                                           //baterija
{
  0b01110,0b11011,0b10001,0b10001,0b10001,0b10001,0b10001,0b11111
};

byte energy[8] =                                                            // munja
{
  0b00010,0b00100,0b01000,0b11111,0b00010,0b00100,0b01000,0b00000
};

byte temperatura[8] =                                                              //termometar
{
 0b00100,0b01010,0b01010,0b01110,0b01110,0b11111,0b11111,0b01110
};

byte pwm[8] =                                                               //simbol pwm 
{
  B11100,B10100,B10101,B10101,B10101,B10101,B00101,B00111
};

// ===============================================================================================
// ===============================================================================================

float citaj_napon(int pina)
{
int x=0;
float n=0;

 n = analogRead(pina);

                                      // funkcija cita i sumira napon, razdelnik je 30k-1k
    n=n*0.00080566406;
    n=n*31.55;
    
 return n;
}

// ===============================================================================================
// ===============================================================================================

float citaj_struju(int pinb)
{
int y=0;
float s=0;

                           // struja merena preko ACS758 BI strujnog hall senzora
     s=analogRead(pinb);
     s = s*0.00080566406;

     s=(s-2.475)*20;
     
     return s;
}

// ===============================================================================================
// ===============================================================================================


int m=0, djuti=1, vduty=1, dt=0;
float  snaga=0, starasnaga=0, vfn=0, starivfn=0, vbat=0, struja=0, starastruja=0, vpod=27.0;                    // podesiti ovde radni napon baterije !!!
double T=0; 
unsigned int refreshcntr=0;

void setup()
{


  thermistor = new NTC_Thermistor(
    SENSOR_PIN,
    REFERENCE_RESISTANCE,
    NOMINAL_RESISTANCE,
    NOMINAL_TEMPERATURE,
    B_VALUE,
    STM32_ANALOG_RESOLUTION // <- for a thermistor calibration
  );



  lcd.begin(COLUMS, ROWS, LCD_5x8DOTS);
  lcd.clear();  
  
    pinMode(PA6, PWM);                          // Ventilator
    pwmWrite(PA6,vduty);

// ===========================================
    RCC_BASE->APB2ENR = 2564;                   // ??? stranica 112 // promenili sa 2076 na 2564 // stranica 112. pdf // 0101000000100b == 2564d
    GPIOA_BASE->CRH = 187;                      // ??? stranica 172 // 
    TIMER1_BASE->CCMR1 = 26744;                 // ??? stranica 349
    TIMER1_BASE->CCER = 17;                     // ??? stranica 353
    TIMER1_BASE->BDTR = 32768;                  // ??? stranica 359
    TIMER1_BASE->ARR = Brojach;                 // ??? stranica 356
    PWM_Vrednost = 1;
// ===========================================


  lcd.createChar(1,solar);
  lcd.createChar(2,battery);
  lcd.createChar(3,energy);
  lcd.createChar(4,temperatura);
  lcd.createChar(5,pwm);

  pinMode(PA0, INPUT_ANALOG);
  pinMode(PA1, INPUT_ANALOG);
  pinMode(PA2, INPUT_ANALOG);
  pinMode(PA3, INPUT_ANALOG);

  
  starivfn=citaj_napon(PA1);
  starastruja=citaj_struju(PA0);
  starasnaga=starivfn*starastruja;
}




void loop()
{

        T=thermistor->readCelsius() ;


      if (T>=50) 
      {
       vduty++; pwmWrite(PA6,vduty);
      } else {vduty--; pwmWrite(PA6,vduty);};


   if (vduty<=1) {vduty=1;};
   if (vduty>=65534) {vduty=65534;};
  
  
// ===========================================

        vbat=citaj_napon(PA2);
        vfn=citaj_napon(PA1);
        struja=citaj_struju(PA0)-0.55;
        snaga=vfn*struja;

// =========================================== 

    if ( (T<60) && (vbat<vpod) && (vfn>=31) && (struja<10.0) )               // podesi idealan napon opterećenja ćelije vfn
    {

          if( snaga>=starasnaga && vfn>=starivfn ) 
           {
              djuti++;
              SetDuty(djuti);
            } 
          

    } else
      {
        djuti--; 
        SetDuty(djuti);
      };                                                                            


    if (djuti<=1) {djuti=1;};                                           // koriguj granicne vrednosti duty-ja.
    if (djuti>=MaxPwmVrednost) {djuti=MaxPwmVrednost;};    

    starivfn=vfn;
    starasnaga=snaga;
    starastruja=struja;

// ===========================================
// ===========================================

   m++;
   if(m>8000)                                                       // ispisuj displej svaki 8000ti prolaz kroz petlju. 
   {
     if(refreshcntr==500) 
     {  
      lcd.clear();
     };

     if(refreshcntr>=600 && refreshcntr<=605) 
     {  
      lcd.setCursor(0,0); lcd.print("MPPT regulator :");
      lcd.setCursor(0,1); lcd.print("Ago 060 195 0000");
      goto kraj;
     };

     if(refreshcntr==606) 
     {  
      lcd.clear();
     };

     
     if(refreshcntr==1000) 
      {  
        lcd.createChar(1,solar);
         lcd.createChar(2,battery);
          lcd.createChar(3,energy);
          lcd.createChar(4,temperatura);
          lcd.createChar(5,pwm);
         lcd.begin(COLUMS, ROWS, LCD_5x8DOTS);  
         lcd.clear();
      refreshcntr=0;
      };

    lcd.setCursor(0,0);                   
    lcd.write(1);
    lcd.setCursor(1,0);
      if (vfn<10) {lcd.setCursor(2,0);};
    lcd.print(vfn,0); lcd.setCursor(3,0); lcd.print("V");

    lcd.setCursor(0,1);
    lcd.write(2);
    lcd.setCursor(1,1); lcd.print(vbat,1); lcd.setCursor(5,1); lcd.print("V");

    lcd.setCursor(5,0);
    lcd.write(3);
    lcd.setCursor(6,0); lcd.print(struja,2); lcd.setCursor(10,0); lcd.print("A"); 

    lcd.setCursor(12,0);
      if(snaga<10.00) { lcd.print("  "); lcd.setCursor(14,0); };
      if((snaga<100.00)&&(snaga>=10)) { lcd.print(" "); lcd.setCursor(13,0); };
    lcd.print(snaga,0); lcd.setCursor(15,0); lcd.print("W"); 

    lcd.setCursor(7,1);
    lcd.write(4);
    lcd.setCursor(8,1);
      if(T<10.00 && T>=0.00) { lcd.print(" "); lcd.setCursor(9,1); };
    lcd.print( T , 0); lcd.setCursor(10,1); lcd.print("C");

    lcd.setCursor(12,1);
    lcd.write(5); dt= map(djuti,1,MaxPwmVrednost,0,100); lcd.setCursor(13,1);

    if (dt<10) { lcd.print(" "); lcd.setCursor(14,1); };
    
    lcd.print(dt); lcd.setCursor(15,1); lcd.print("%");

kraj:
     
      refreshcntr++;
    } 


  if (m>8000) {m=0;};                                                  // 8000ti prolaz kroz petlju

}
